@@ -1,15 +1,14 @@
-#!/usr/bin/python
-# Copyright 2010 Gentoo Foundation
+#!/usr/bin/python -b
+# Copyright 2010-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 #
 # This is a helper which ebuild processes can use
 # to communicate with portage's main python process.
 
-import array
 import logging
 import os
 import pickle
-import select
+import platform
 import signal
 import sys
 import time
@@ -17,7 +16,13 @@ import time
 def debug_signal(signum, frame):
 	import pdb
 	pdb.set_trace()
-signal.signal(signal.SIGUSR1, debug_signal)
+
+if platform.python_implementation() == 'Jython':
+	debug_signum = signal.SIGUSR2 # bug #424259
+else:
+	debug_signum = signal.SIGUSR1
+
+signal.signal(debug_signum, debug_signal)
 
 # Avoid sandbox violations after python upgrade.
 pym_path = os.path.join(os.path.dirname(
@@ -30,14 +35,28 @@ if os.environ.get("SANDBOX_ON") == "1":
 			":".join(filter(None, sandbox_write))
 
 import portage
+portage._internal_caller = True
 portage._disable_legacy_globals()
 
+from portage.util._async.ForkProcess import ForkProcess
+from portage.util._eventloop.global_event_loop import global_event_loop
+from _emerge.PipeReader import PipeReader
+
+class FifoWriter(ForkProcess):
+
+	__slots__ = ('buf', 'fifo',)
+
+	def _run(self):
+		# Atomically write the whole buffer into the fifo.
+		with open(self.fifo, 'wb', 0) as f:
+			f.write(self.buf)
+		return os.EX_OK
+
 class EbuildIpc(object):
 
 	# Timeout for each individual communication attempt (we retry
 	# as long as the daemon process appears to be alive).
-	_COMMUNICATE_RETRY_TIMEOUT_SECONDS = 15
-	_BUFSIZE = 4096
+	_COMMUNICATE_RETRY_TIMEOUT_MS = 15000
 
 	def __init__(self):
 		self.fifo_dir = os.environ['PORTAGE_BUILDDIR']
@@ -81,7 +100,7 @@ class EbuildIpc(object):
 			'ebuild-ipc: daemon process not detected\n'),
 			level=logging.ERROR, noiselevel=-1)
 
-	def _wait(self, pid, pr, msg):
+	def _run_writer(self, fifo_writer, msg):
 		"""
 		Wait on pid and return an appropriate exit code. This
 		may return unsuccessfully due to timeout if the daemon
@@ -90,70 +109,48 @@ class EbuildIpc(object):
 
 		start_time = time.time()
 
-		while True:
-			try:
-				events = select.select([pr], [], [],
-					self._COMMUNICATE_RETRY_TIMEOUT_SECONDS)
-			except select.error as e:
-				portage.util.writemsg_level(
-					"ebuild-ipc: %s: %s\n" % \
-					(portage.localization._('during select'), e),
-					level=logging.ERROR, noiselevel=-1)
-				continue
+		fifo_writer.start()
+		eof = fifo_writer.poll() is not None
 
-			if events[0]:
+		while not eof:
+			fifo_writer._wait_loop(timeout=self._COMMUNICATE_RETRY_TIMEOUT_MS)
+
+			eof = fifo_writer.poll() is not None
+			if eof:
 				break
-
-			if self._daemon_is_alive():
+			elif self._daemon_is_alive():
 				self._timeout_retry_msg(start_time, msg)
 			else:
+				fifo_writer.cancel()
 				self._no_daemon_msg()
-				try:
-					os.kill(pid, signal.SIGKILL)
-					os.waitpid(pid, 0)
-				except OSError as e:
-					portage.util.writemsg_level(
-						"ebuild-ipc: %s\n" % (e,),
-						level=logging.ERROR, noiselevel=-1)
+				fifo_writer.wait()
 				return 2
 
-		try:
-			wait_retval = os.waitpid(pid, 0)
-		except OSError as e:
-			portage.util.writemsg_level(
-				"ebuild-ipc: %s: %s\n" % (msg, e),
-				level=logging.ERROR, noiselevel=-1)
-			return 2
+		return fifo_writer.wait()
 
-		if not os.WIFEXITED(wait_retval[1]):
-			portage.util.writemsg_level(
-				"ebuild-ipc: %s: %s\n" % (msg,
-				portage.localization._('subprocess failure: %s') % \
-				wait_retval[1]),
-				level=logging.ERROR, noiselevel=-1)
-			return 2
+	def _receive_reply(self, input_fd):
 
-		return os.WEXITSTATUS(wait_retval[1])
+		start_time = time.time()
 
-	def _receive_reply(self):
+		pipe_reader = PipeReader(input_files={"input_fd":input_fd},
+			scheduler=global_event_loop())
+		pipe_reader.start()
 
-		# File streams are in unbuffered mode since we do atomic
-		# read and write of whole pickles.
-		input_file = open(self.ipc_out_fifo, 'rb', 0)
+		eof = pipe_reader.poll() is not None
 
-		# For maximum portability, us an array in order to force
-		# a single atomic read of a whole pickle (bug #337465).
-		buf = array.array('B')
+		while not eof:
+			pipe_reader._wait_loop(timeout=self._COMMUNICATE_RETRY_TIMEOUT_MS)
+			eof = pipe_reader.poll() is not None
+			if not eof:
+				if self._daemon_is_alive():
+					self._timeout_retry_msg(start_time,
+						portage.localization._('during read'))
+				else:
+					pipe_reader.cancel()
+					self._no_daemon_msg()
+					return 2
 
-		try:
-			buf.fromfile(input_file, self._BUFSIZE)
-		except (EOFError, IOError) as e:
-			if not buf:
-				portage.util.writemsg_level(
-					"ebuild-ipc: %s\n" % (e,),
-					level=logging.ERROR, noiselevel=-1)
-
-		input_file.close()
+		buf = pipe_reader.getvalue()
 
 		retval = 2
 
@@ -167,7 +164,7 @@ class EbuildIpc(object):
 		else:
 
 			try:
-				reply = pickle.loads(buf.tostring())
+				reply = pickle.loads(buf)
 			except SystemExit:
 				raise
 			except Exception as e:
@@ -195,28 +192,20 @@ class EbuildIpc(object):
 			self._no_daemon_msg()
 			return 2
 
+		# Open the input fifo before the output fifo, in order to make it
+		# possible for the daemon to send a reply without blocking. This
+		# improves performance, and also makes it possible for the daemon
+		# to do a non-blocking write without a race condition.
+		input_fd = os.open(self.ipc_out_fifo,
+			os.O_RDONLY|os.O_NONBLOCK)
+
 		# Use forks so that the child process can handle blocking IO
 		# un-interrupted, while the parent handles all timeout
 		# considerations. This helps to avoid possible race conditions
 		# from interference between timeouts and blocking IO operations.
-		pr, pw = os.pipe()
-		pid = os.fork()
-
-		if pid == 0:
-			os.close(pr)
-
-			# File streams are in unbuffered mode since we do atomic
-			# read and write of whole pickles.
-			output_file = open(self.ipc_in_fifo, 'wb', 0)
-			output_file.write(pickle.dumps(args))
-			output_file.close()
-			os._exit(os.EX_OK)
-
-		os.close(pw)
-
 		msg = portage.localization._('during write')
-		retval = self._wait(pid, pr, msg)
-		os.close(pr)
+		retval = self._run_writer(FifoWriter(buf=pickle.dumps(args),
+			fifo=self.ipc_in_fifo, scheduler=global_event_loop()), msg)
 
 		if retval != os.EX_OK:
 			portage.util.writemsg_level(
@@ -229,18 +218,7 @@ class EbuildIpc(object):
 			self._no_daemon_msg()
 			return 2
 
-		pr, pw = os.pipe()
-		pid = os.fork()
-
-		if pid == 0:
-			os.close(pr)
-			retval = self._receive_reply()
-			os._exit(retval)
-
-		os.close(pw)
-		retval = self._wait(pid, pr, portage.localization._('during read'))
-		os.close(pr)
-		return retval
+		return self._receive_reply(input_fd)
 
 def ebuild_ipc_main(args):
 	ebuild_ipc = EbuildIpc()
