@@ -1,5 +1,5 @@
 #!/bin/bash
-# Copyright 1999-2012 Gentoo Foundation
+# Copyright 1999-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 export DESTTREE=/usr
@@ -11,6 +11,8 @@ export EXEOPTIONS="-m0755"
 export LIBOPTIONS="-m0644"
 export DIROPTIONS="-m0755"
 export MOPREFIX=${PN}
+# Do not compress files which are smaller than this (in bytes). #169260
+export PORTAGE_DOCOMPRESS_SIZE_LIMIT="128"
 declare -a PORTAGE_DOCOMPRESS=( /usr/share/{doc,info,man} )
 declare -a PORTAGE_DOCOMPRESS_SKIP=( /usr/share/doc/${PF}/html )
 
@@ -141,30 +143,6 @@ docompress() {
 	fi
 }
 
-# adds ".keep" files so that dirs aren't auto-cleaned
-keepdir() {
-	dodir "$@"
-	local x
-	if ! ___eapi_has_prefix_variables; then
-		local ED=${D}
-	fi
-	if [ "$1" == "-R" ] || [ "$1" == "-r" ]; then
-		shift
-		find "$@" -type d -printf "${ED}%p/.keep_${CATEGORY}_${PN}-${SLOT%/*}\n" \
-			| tr "\n" "\0" | \
-			while read -r -d $'\0' ; do
-				>> "$REPLY" || \
-					die "Failed to recursively create .keep files"
-			done
-	else
-		for x in "$@"; do
-			>> "${ED}${x}/.keep_${CATEGORY}_${PN}-${SLOT%/*}" || \
-				die "Failed to create .keep in ${ED}${x}"
-		done
-	fi
-}
-
-
 useq() {
 	has $EBUILD_PHASE prerm postrm || eqawarn \
 		"QA Notice: The 'useq' function is deprecated (replaced by 'use')"
@@ -210,11 +188,20 @@ use() {
 		#fi
 		true
 
-	# Make sure we have this USE flag in IUSE
-	elif [[ -n $PORTAGE_IUSE && -n $EBUILD_PHASE ]] ; then
-		[[ $u =~ $PORTAGE_IUSE ]] || \
+	# Make sure we have this USE flag in IUSE, but exempt binary
+	# packages for API consumers like Entropy which do not require
+	# a full profile with IUSE_IMPLICIT and stuff (see bug #456830).
+	elif [[ -n $PORTAGE_IUSE && -n $EBUILD_PHASE &&
+		-n $PORTAGE_INTERNAL_CALLER ]] ; then
+		if [[ ! $u =~ $PORTAGE_IUSE ]] ; then
+			if [[ ! ${EAPI} =~ ^(0|1|2|3|4|4-python|4-slot-abi)$ ]] ; then
+				# This is only strict starting with EAPI 5, since implicit IUSE
+				# is not well defined for earlier EAPIs (see bug #449708).
+				die "USE Flag '${u}' not in IUSE for ${CATEGORY}/${PF}"
+			fi
 			eqawarn "QA Notice: USE Flag '${u}' not" \
 				"in IUSE for ${CATEGORY}/${PF}"
+		fi
 	fi
 
 	local IFS=$' \t\n' prev_shopts=$- ret
@@ -275,8 +262,8 @@ use_enable() {
 unpack() {
 	local srcdir
 	local x
-	local y
-	local suffix
+	local y y_insensitive
+	local suffix suffix_insensitive
 	local myfail
 	local eapi=${EAPI:-0}
 	[ -z "$*" ] && die "Nothing passed to the 'unpack' command"
@@ -284,10 +271,10 @@ unpack() {
 	for x in "$@"; do
 		__vecho ">>> Unpacking ${x} to ${PWD}"
 		suffix=${x##*.}
-		suffix=$(LC_ALL=C tr "[:upper:]" "[:lower:]" <<< "${suffix}")
+		suffix_insensitive=$(LC_ALL=C tr "[:upper:]" "[:lower:]" <<< "${suffix}")
 		y=${x%.*}
 		y=${y##*.}
-		y=$(LC_ALL=C tr "[:upper:]" "[:lower:]" <<< "${y}")
+		y_insensitive=$(LC_ALL=C tr "[:upper:]" "[:lower:]" <<< "${y}")
 
 		if [[ ${x} == "./"* ]] ; then
 			srcdir=""
@@ -301,7 +288,13 @@ unpack() {
 		[[ ! -s ${srcdir}${x} ]] && die "${x} does not exist"
 
 		__unpack_tar() {
-			if [ "${y}" == "tar" ]; then
+			if [[ ${y_insensitive} == tar ]] ; then
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ tar != ${y} ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"secondary suffix '${y}' which is unofficially" \
+						"supported with EAPI '${EAPI}'. Instead use 'tar'."
+				fi
 				$1 -c -- "$srcdir$x" | tar xof -
 				__assert_sigpipe_ok "$myfail"
 			else
@@ -312,27 +305,64 @@ unpack() {
 		}
 
 		myfail="failure unpacking ${x}"
-		case "${suffix}" in
+		case "${suffix_insensitive}" in
 			tar)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ tar != ${suffix} ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'tar'."
+				fi
 				tar xof "$srcdir$x" || die "$myfail"
 				;;
 			tgz)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ tgz != ${suffix} ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'tgz'."
+				fi
 				tar xozf "$srcdir$x" || die "$myfail"
 				;;
 			tbz|tbz2)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " tbz tbz2 " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'tbz' or 'tbz2'."
+				fi
 				${PORTAGE_BUNZIP2_COMMAND:-${PORTAGE_BZIP2_COMMAND} -d} -c -- "$srcdir$x" | tar xof -
 				__assert_sigpipe_ok "$myfail"
 				;;
 			zip|jar)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " ZIP zip jar " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'." \
+						"Instead use 'ZIP', 'zip', or 'jar'."
+				fi
 				# unzip will interactively prompt under some error conditions,
 				# as reported in bug #336285
 				( set +x ; while true ; do echo n || break ; done ) | \
 				unzip -qo "${srcdir}${x}" || die "$myfail"
 				;;
 			gz|z)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " gz z Z " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'gz', 'z', or 'Z'."
+				fi
 				__unpack_tar "gzip -d"
 				;;
 			bz2|bz)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " bz bz2 " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'bz' or 'bz2'."
+				fi
 				__unpack_tar "${PORTAGE_BUNZIP2_COMMAND:-${PORTAGE_BZIP2_COMMAND} -d}"
 				;;
 			7z)
@@ -344,15 +374,40 @@ unpack() {
 				fi
 				;;
 			rar)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " rar RAR " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'rar' or 'RAR'."
+				fi
 				unrar x -idq -o+ "${srcdir}${x}" || die "$myfail"
 				;;
 			lha|lzh)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " LHA LHa lha lzh " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'." \
+						"Instead use 'LHA', 'LHa', 'lha', or 'lzh'."
+				fi
 				lha xfq "${srcdir}${x}" || die "$myfail"
 				;;
 			a)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " a " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'a'."
+				fi
 				ar x "${srcdir}${x}" || die "$myfail"
 				;;
 			deb)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " deb " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'deb'."
+				fi
 				# Unpacking .deb archives can not always be done with
 				# `ar`.  For instance on AIX this doesn't work out.  If
 				# we have `deb2targz` installed, prefer it over `ar` for
@@ -380,9 +435,21 @@ unpack() {
 				fi
 				;;
 			lzma)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " lzma " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'lzma'."
+				fi
 				__unpack_tar "lzma -d"
 				;;
 			xz)
+				if ___eapi_unpack_is_case_sensitive && \
+					[[ " xz " != *" ${suffix} "* ]] ; then
+					eqawarn "QA Notice: unpack called with" \
+						"suffix '${suffix}' which is unofficially supported" \
+						"with EAPI '${EAPI}'. Instead use 'xz'."
+				fi
 				if ___eapi_unpack_supports_xz; then
 					__unpack_tar "xz -d"
 				else
@@ -402,6 +469,7 @@ unpack() {
 
 econf() {
 	local x
+	local pid=${BASHPID:-$(__bashpid)}
 
 	if ! ___eapi_has_prefix_variables; then
 		local EPREFIX=
@@ -433,15 +501,23 @@ econf() {
 	if [ -x "${ECONF_SOURCE}/configure" ]; then
 		if [[ -n $CONFIG_SHELL && \
 			"$(head -n1 "$ECONF_SOURCE/configure")" =~ ^'#!'[[:space:]]*/bin/sh([[:space:]]|$) ]] ; then
-			sed -e "1s:^#![[:space:]]*/bin/sh:#!$CONFIG_SHELL:" -i "$ECONF_SOURCE/configure" || \
-				die "Substition of shebang in '$ECONF_SOURCE/configure' failed"
+			# preserve timestamp, see bug #440304
+			touch -r "${ECONF_SOURCE}/configure" "${ECONF_SOURCE}/configure._portage_tmp_.${pid}" || die
+			sed -i \
+				-e "1s:^#![[:space:]]*/bin/sh:#!$CONFIG_SHELL:" \
+				"${ECONF_SOURCE}/configure" \
+				|| die "Substition of shebang in '${ECONF_SOURCE}/configure' failed"
+			touch -r "${ECONF_SOURCE}/configure._portage_tmp_.${pid}" "${ECONF_SOURCE}/configure" || die
+			rm -f "${ECONF_SOURCE}/configure._portage_tmp_.${pid}"
 		fi
 		if [ -e "${EPREFIX}"/usr/share/gnuconfig/ ]; then
 			find "${WORKDIR}" -type f '(' \
 			-name config.guess -o -name config.sub ')' -print0 | \
 			while read -r -d $'\0' x ; do
 				__vecho " * econf: updating ${x/${WORKDIR}\/} with ${EPREFIX}/usr/share/gnuconfig/${x##*/}"
-				cp -f "${EPREFIX}"/usr/share/gnuconfig/"${x##*/}" "${x}"
+				# Make sure we do this atomically incase we're run in parallel. #487478
+				cp -f "${EPREFIX}"/usr/share/gnuconfig/"${x##*/}" "${x}.${pid}"
+				mv -f "${x}.${pid}" "${x}"
 			done
 		fi
 
@@ -481,6 +557,9 @@ econf() {
 			set -- --libdir="$(__strip_duplicate_slashes "${CONF_PREFIX}${CONF_LIBDIR}")" "$@"
 		fi
 
+		# Handle arguments containing quoted whitespace (see bug #457136).
+		eval "local -a EXTRA_ECONF=(${EXTRA_ECONF})"
+
 		set -- \
 			--prefix="${EPREFIX}"/usr \
 			${CBUILD:+--build=${CBUILD}} \
@@ -492,7 +571,7 @@ econf() {
 			--sysconfdir="${EPREFIX}"/etc \
 			--localstatedir="${EPREFIX}"/var/lib \
 			"$@" \
-			${EXTRA_ECONF}
+			"${EXTRA_ECONF[@]}"
 		__vecho "${ECONF_SOURCE}/configure" "$@"
 
 		if ! "${ECONF_SOURCE}/configure" "$@" ; then
@@ -660,15 +739,21 @@ has_version() {
 	fi
 
 	if ___eapi_has_prefix_variables; then
-		eroot=${root%/}${EPREFIX}/
+		# [[ ${root} == / ]] would be ambiguous here,
+		# since both prefixes can share root=/ while
+		# having different EPREFIX offsets.
+		if ${host_root} ; then
+			eroot=${root%/}${PORTAGE_OVERRIDE_EPREFIX}/
+		else
+			eroot=${root%/}${EPREFIX}/
+		fi
 	else
 		eroot=${root}
 	fi
 	if [[ -n $PORTAGE_IPC_DAEMON ]] ; then
 		"$PORTAGE_BIN_PATH"/ebuild-ipc has_version "${eroot}" "${atom}"
 	else
-		PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
-		"${PORTAGE_PYTHON:-/usr/bin/python}" "${PORTAGE_BIN_PATH}/portageq" has_version "${eroot}" "${atom}"
+		"${PORTAGE_BIN_PATH}/ebuild-helpers/portageq" has_version "${eroot}" "${atom}"
 	fi
 	local retval=$?
 	case "${retval}" in
@@ -713,15 +798,21 @@ best_version() {
 	fi
 
 	if ___eapi_has_prefix_variables; then
-		eroot=${root%/}${EPREFIX}/
+		# [[ ${root} == / ]] would be ambiguous here,
+		# since both prefixes can share root=/ while
+		# having different EPREFIX offsets.
+		if ${host_root} ; then
+			eroot=${root%/}${PORTAGE_OVERRIDE_EPREFIX}/
+		else
+			eroot=${root%/}${EPREFIX}/
+		fi
 	else
 		eroot=${root}
 	fi
 	if [[ -n $PORTAGE_IPC_DAEMON ]] ; then
 		"$PORTAGE_BIN_PATH"/ebuild-ipc best_version "${eroot}" "${atom}"
 	else
-		PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
-		"${PORTAGE_PYTHON:-/usr/bin/python}" "${PORTAGE_BIN_PATH}/portageq" best_version "${eroot}" "${atom}"
+		"${PORTAGE_BIN_PATH}/ebuild-helpers/portageq" best_version "${eroot}" "${atom}"
 	fi
 	local retval=$?
 	case "${retval}" in
@@ -750,8 +841,7 @@ if ___eapi_has_master_repositories; then
 		if [[ -n ${PORTAGE_IPC_DAEMON} ]]; then
 			"${PORTAGE_BIN_PATH}/ebuild-ipc" master_repositories "${EROOT}" "${repository}"
 		else
-			output=$(PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
-			"${PORTAGE_PYTHON:-/usr/bin/python}" "${PORTAGE_BIN_PATH}/portageq" master_repositories "${EROOT}" "${repository}")
+			output=$("${PORTAGE_BIN_PATH}/ebuild-helpers/portageq" master_repositories "${EROOT}" "${repository}")
 		fi
 		retval=$?
 		[[ -n ${output} ]] && echo "${output}"
@@ -782,8 +872,7 @@ if ___eapi_has_repository_path; then
 		if [[ -n ${PORTAGE_IPC_DAEMON} ]]; then
 			"${PORTAGE_BIN_PATH}/ebuild-ipc" repository_path "${EROOT}" "${repository}"
 		else
-			output=$(PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
-			"${PORTAGE_PYTHON:-/usr/bin/python}" "${PORTAGE_BIN_PATH}/portageq" get_repo_path "${EROOT}" "${repository}")
+			output=$("${PORTAGE_BIN_PATH}/ebuild-helpers/portageq" get_repo_path "${EROOT}" "${repository}")
 		fi
 		retval=$?
 		[[ -n ${output} ]] && echo "${output}"
@@ -813,8 +902,7 @@ if ___eapi_has_available_eclasses; then
 		if [[ -n ${PORTAGE_IPC_DAEMON} ]]; then
 			"${PORTAGE_BIN_PATH}/ebuild-ipc" available_eclasses "${EROOT}" "${repository}"
 		else
-			output=$(PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
-			"${PORTAGE_PYTHON:-/usr/bin/python}" "${PORTAGE_BIN_PATH}/portageq" available_eclasses "${EROOT}" "${repository}")
+			output=$("${PORTAGE_BIN_PATH}/ebuild-helpers/portageq" available_eclasses "${EROOT}" "${repository}")
 		fi
 		retval=$?
 		[[ -n ${output} ]] && echo "${output}"
@@ -845,8 +933,7 @@ if ___eapi_has_eclass_path; then
 		if [[ -n ${PORTAGE_IPC_DAEMON} ]]; then
 			"${PORTAGE_BIN_PATH}/ebuild-ipc" eclass_path "${EROOT}" "${repository}" "${eclass}"
 		else
-			output=$(PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
-			"${PORTAGE_PYTHON:-/usr/bin/python}" "${PORTAGE_BIN_PATH}/portageq" eclass_path "${EROOT}" "${repository}" "${eclass}")
+			output=$("${PORTAGE_BIN_PATH}/ebuild-helpers/portageq" eclass_path "${EROOT}" "${repository}" "${eclass}")
 		fi
 		retval=$?
 		[[ -n ${output} ]] && echo "${output}"
@@ -877,8 +964,7 @@ if ___eapi_has_license_path; then
 		if [[ -n ${PORTAGE_IPC_DAEMON} ]]; then
 			"${PORTAGE_BIN_PATH}/ebuild-ipc" license_path "${EROOT}" "${repository}" "${license}"
 		else
-			output=$(PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
-			"${PORTAGE_PYTHON:-/usr/bin/python}" "${PORTAGE_BIN_PATH}/portageq" license_path "${EROOT}" "${repository}" "${license}")
+			output=$("${PORTAGE_BIN_PATH}/ebuild-helpers/portageq" license_path "${EROOT}" "${repository}" "${license}")
 		fi
 		retval=$?
 		[[ -n ${output} ]] && echo "${output}"
