@@ -1,4 +1,4 @@
-# Copyright 2010-2013 Gentoo Foundation
+# Copyright 2010-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from __future__ import print_function
@@ -347,7 +347,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0,
 				_("!!! For fetching to a read-only filesystem, "
 				"locking should be turned off.\n")), noiselevel=-1)
 			writemsg(_("!!! This can be done by adding -distlocks to "
-				"FEATURES in /etc/make.conf\n"), noiselevel=-1)
+				"FEATURES in /etc/portage/make.conf\n"), noiselevel=-1)
 #			use_locks = 0
 
 	# local mirrors are always added
@@ -445,8 +445,9 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0,
 					filedict[myfile].extend(uris)
 					thirdpartymirror_uris.setdefault(myfile, []).extend(uris)
 
-				if not filedict[myfile]:
-					writemsg(_("No known mirror by the name: %s\n") % (mirrorname))
+				if mirrorname not in custommirrors and \
+					mirrorname not in thirdpartymirrors:
+					writemsg(_("!!! No known mirror by the name: %s\n") % (mirrorname))
 			else:
 				writemsg(_("Invalid mirror definition in SRC_URI:\n"), noiselevel=-1)
 				writemsg("  %s\n" % (myuri), noiselevel=-1)
@@ -752,7 +753,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0,
 							"remaining space.\n"), noiselevel=-1)
 						if userfetch:
 							writemsg(_("!!! You may set FEATURES=\"-userfetch\""
-								" in /etc/make.conf in order to fetch with\n"
+								" in /etc/portage/make.conf in order to fetch with\n"
 								"!!! superuser privileges.\n"), noiselevel=-1)
 
 				if fsmirrors and not os.path.exists(myfile_path) and has_space:
@@ -865,8 +866,8 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0,
 				protocol = loc[0:loc.find("://")]
 
 				global_config_path = GLOBAL_CONFIG_PATH
-				if mysettings['EPREFIX']:
-					global_config_path = os.path.join(mysettings['EPREFIX'],
+				if portage.const.EPREFIX:
+					global_config_path = os.path.join(portage.const.EPREFIX,
 							GLOBAL_CONFIG_PATH.lstrip(os.sep))
 
 				missing_file_param = False
